from unittest import TestCase
from Core.command import Command
from Core.engine import CommandEngine
from Core.receiver import Receiver
from Core.recording import ReceiverRecording
import engine_tests_data


class EngineFixture(TestCase):
    
    def setUp(self):
        self._subscriber = ReceiverRecording()
          

    def test_execute(self):
        command = engine_tests_data.MyTestCommand()
        receiver = engine_tests_data.MyTestHandler()
        self._subscriber.record(engine_tests_data.MyTestCommand, lambda:receiver)
        self._command_engine = CommandEngine(self._subscriber)
        self._command_engine.process(command)
        self.assertEqual(receiver._mvalue, "modified value")
    
    def test_executeWithLogging(self):
        command = engine_tests_data.MyTestCommand()
        receiverLog = engine_tests_data.MyTestHandlerWithLog()
        self._subscriber.record(engine_tests_data.MyTestCommand, lambda:receiverLog)
        self._command_engine = CommandEngine(self._subscriber)
        self._command_engine.process(command)
        self.assertEqual(receiverLog._mvalue, "modified log value")




import logging
from Core.command import Command
from Core.receiver import Receiver
from Core.request import Request
from Core.receiver_log import receiver_log, OutputType

class MyTestHandler(Receiver):
    def __init__(self):
        self._mvalue = "base value"
    
    
    def execute(self, request:Request):
        self._mvalue = "modified value"
        

class MyTestHandlerWithLog(Receiver):
    def __init__(self):
        self._mvalue = "base log value"
    
    @receiver_log(logging.DEBUG, OutputType.file_output)
    def execute(self, request:Request):
        self._mvalue = "modified log value"


class MyTestCommand(Command):
    def __init__(self):
        super().__init__()
    
    



from uuid import UUID, uuid4
from abc import ABCMeta, abstractmethod



class Request(metaclass=ABCMeta):
    
    def __init__(self) -> None:
        self._requestid = uuid4()
    
    @property
    def id(self) -> UUID:
        return self._requestid
    
    @staticmethod
    def is_aCommand() -> bool:
        return False
    



from functools import wraps
import logging
from Core.request import Request
from Core.exceptions import ConfigException
from enum import Enum

DEFAULTSTARTMSG = "Starting operation {}"
DEFAULTENDMSG = "Ending operation {}"

class OutputType(Enum):
    console_output = 1
    file_output = 2



def receiver_log(level=logging.DEBUG, output_mode:OutputType=1, log_name:str=None, start_msg:str=None, end_msg:str=None):
    def func_decorator(func):
        @wraps(func)
        def func_wrapper(*args, **kwargs):
            name_log = log_name if log_name else func.__module__
            log = logging.getLogger(name_log)
            start_log_msg = start_msg if start_msg else DEFAULTSTARTMSG.format(func.__name__)
            end_log_msg  = end_msg if end_msg else DEFAULTENDMSG.format(func.__name__)
            req = args[1]
            if not isinstance(req, Request):
                raise ConfigException("Request not found")
            
            request_name = " {}".format(str(req))
            if output_mode == 2:
                handler = logging.FileHandler('amuru.log')
                handler.setLevel(level)
                formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
                handler.setFormatter(formatter)
                log.addHandler(handler)

            log.log(level, start_log_msg + request_name)
            result = func(*args, **kwargs)
            log.log(level, start_log_msg + request_name)

            return result
        return func_wrapper
    return func_decorator
            

            





from typing import Optional
from Core.command import Command
from Core.recording import ReceiverRecording
from Core.exceptions import EngineException


class CommandEngine(object):

    def __init__(self, receiver_factory:Optional[ReceiverRecording]=None):
        self._receiver_factory = receiver_factory
    
    def process(self, my_command:Command) -> None:
       if not self._receiver_factory:
           raise EngineException("no factory found")
       receiver = self._receiver_factory.resolve(my_command)
       receiver_object = receiver()
       receiver_object.execute(my_command)
        
    

    




from setuptools import setup, find_packages

setup(
    name='Amuru',
    description='a tiny command invoker/dispatcher pattern library ',
    long_description="""See the Github project page (https://github.com/thynquest/amuru.git) for more on information""",
    license='MIT',
    keywords="command invoker/dispatcher pattern library",
    version='1.0.0',
    author='Franck Kambiwa',
    author_email='thynquest@gmail.com',
    url='https://github.com/thynquest/amuru.git',

    packages=find_packages(),
    install_requires=['ez_setup'],
        
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.6",
        "License :: OSI Approved :: MIT License",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Topic :: System :: Monitoring",
        "Topic :: System :: Distributed Computing",
    ]
)



from typing import Callable
from Core.receiver import Receiver
from Core.request import Request
from Core.command import Command
from Core.exceptions import EngineException



class ReceiverRecording(object):
    def __init__(self) -> None:
        self._recording = {}

    
    def record(self, request:Command, implementation:Callable[[], Receiver]) -> None:
        key = request.__name__
        if request.is_aCommand() and key in self._recording.keys():
            raise RecordingException("this command with the same name already exist")
        self._recording[key] = implementation
    

    def resolve(self, request_object:Command) -> Callable[[], Receiver]:
        key = request_object.__class__.__name__
        if key not in self._recording.keys():
            raise RecordingException("no receiver found for this command")
        
        return self._recording[key]

    
    

    
    
        